		#ifndef EIGENH
		#define EIGENH


/**
 Realiza la transformacin Householder
 A es de tamao nxn, 
 v es un vector de tamao n 
**/
double houseHolder( double **A,  int n, int k, double *v );

/**
   Funcin que reduce la matriz A, de tamao n x n,
 a una matriz tridiagonal. La matriz de salida se sobreescribe en A
**/
void tridiagonal( double **A, int n, double *vec );


/**
# Rutina que multiplica A^T * A
# Solo es necesario calcular la porcin superior (o inferior)
# de la matriz dado que el resultado es una matriz simtrica
# Args: S = E^T * E, y n, el tamao de las matrices es n x n
**/
void multiplicaATA( double **S, double **E, int n );

/**
# Rutina que encuentra los eigenvalores de una matriz tridiagonal
# Args: M, la matriz tridiagonal de entrada
#       n, el tamao de la matriz de n x n
#      eigen, referencia para almacenar los eigenvalores
#      m1, m2, los eigenvalores deseados, l_m1 < l_m1+1 < l_m1+2 < ... < l_m2
**/
void biseccion( double **M, int n, double *eigen, int m1, int m2, double *vec );

/**
  Subrutina que calcula descomposicin QR
  A = QR, A es de tamao m x n
  No retorna el valor de  Q
**/
void descomposicionQR( double **A, int m, int n, double **R, double *vec );

/**
 Si se tiene el sistema Ax = b, A es de tamao n x n, 
se realiza la matriz aumentada [A|b] y se realiza
la descomposicin QR, se obtiene Q[ R | Q^T b ]
Esta rutina obtiene x a partir de la matriz Rp = [ R | Q^T b ]
**/
void resuelve( double **Rp, int n, double *x );

#endif
