#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <sys/time.h>
#include "matrix.h"
#include "eigen.h"

/**
 Rutina que lee la matriz de entrada almacenada en un archivo
 Args: nombre_archivo referencia_a_la_matriz_de_salida
**/
int lee_archivo( char *nombrearchivo, double A[][5], int *mynn ) 
{
	char linea[80];
	FILE *arch;
	int i, j, k;
	double val;
	char *ptr;

	arch = fopen( nombrearchivo, "r" );
	if( arch == NULL ) {
		fprintf( stderr, "No se puede abrir archivo %s\n", nombrearchivo );
		return( -1 );
	}

	for( i=0; ; i++ ){
		if( fgets( linea, 80, arch ) == NULL ) break;
		ptr = linea;
		for( j=0; j<5; j++ ) {
			k = sscanf( ptr, "%lf", &val );
			if ( k == 1 ) {
				// printf( "%d %d %d\n", i, j, k );
				A[i][j] = val;
				while( *ptr != ' ' && *ptr != '\n' ) ptr++;
				ptr++;
				// printf("%s\n", ptr );
			}
		}
	}
	fclose( arch );
	*mynn = i;
	return( 0 );
}

/**
	Imprime una matrix A de tamao n x n
**/
void imprime( double **A, int n )
{
	int i, j;

	for( i=0; i<n; i++ ) {
		for( j=0; j<n; j++ ) {
			printf( "%lf ", A[i][j] );
		}
		printf("\n");
	}
	printf("\n");
}


int main( int argc, char **argv )
{
	int i, j, k, itera;
	char nombre[32];
	MATRIX *M2, *M3, *M4;
	double  **Bcopia, **R;
	int n;
	double eigen[4];
	double *vec, suma;
	double datos[1000][5];
	double **A, **B;
	double w[12];
	double x, y, val;
	double *tmp1, *tmp2;
	struct timeval  tv1, tv2;

	if( argc != 2 ) {
		fprintf( stderr, "Args: archivo\n" );
		return( 1 );
	}

	strncpy( nombre, argv[1], 32 );

	if ( lee_archivo( nombre, datos, &n ) != 0 ) {
		fprintf( stderr, "ERROR leyendo archivo de entrada\n" );
		return( 2 );
	}
	printf( "Lei %d datos\n", n );
	M2 = matrix_allocate( 12, 12, sizeof(double) );
	if( M2==NULL ) {
		fprintf( stderr, "ERROR no hay memoria para la matriz M2\n" );
		return( 4 );
	}
	B = (double **)M2->ptr;
				
		
	// Hago una copia de la matriz original
	M3 = matrix_allocate( 12, 13, sizeof(double) );
	if( M3==NULL ) {
		matrix_free( M2 );
		fprintf( stderr, "ERROR no hay memoria para la matriz M3\n" );
		return( 4 );
	}
	Bcopia = (double **) M3->ptr;
	A = (double **) M3->ptr;

	// Memoria para un vector auxiliar de tamao 3*n
	vec = (double *)malloc( 3*12*sizeof(double) );
	if( vec==NULL ) {
		fprintf( stderr, "ERROR no hay memoria para el vector vec\n" );
		matrix_free( M2 );
		matrix_free( M3 );
		return( 5 );
	}
	// Memoria para la matriz R
	M4 = matrix_allocate( 13, 13, sizeof(double) );
	if( M4==NULL ) {
		fprintf( stderr, "ERROR no hay memoria para la matriz M4\n" );
		matrix_free( M2 );
		matrix_free( M3 );
		free(vec);
		return( 6 );
	}
	R = (double **)M4->ptr;

	/**
	 El cdigo!
     El mtodo que tengo que compara es:

    1. Contruir B = A^T * A
	2. Calcular el eigenvector correspondiente el menor eiganvalor, distinto de cero, de B
	3. Repetir esto... 1000 veces?
	**/
	gettimeofday(&tv1, NULL);
  for( itera=0; itera<1; itera++ ) {

	// Limpio la mitad superior de la matrix B
	for( i=0; i<12; i++ ){
		tmp1 = &B[i][i];
		for( j=i; j<12; j++ )
			*tmp1++ = 0.0;
	}
	// Limpio toda la matrix A
	for( i=0; i<12; i++ ){
		tmp1 = A[i];
		for( j=0; j<12; j++ )
			*tmp1++ = 0.0;
	}

	for( k=0; k<n; k++ ) {

		// printf("%lf %lf %lf %lf %lf\n", datos[k][0], datos[k][1], datos[k][2], datos[k][3], datos[k][4] );
		x = datos[k][3];
		y = datos[k][4];

		w[4] = -datos[k][0];
		w[5] = -datos[k][1];
		w[6] = -datos[k][2];
		w[7] = -1.0;
		w[8] =  datos[k][0]*y;
		w[9] =  datos[k][1]*y;
		w[10] = datos[k][2]*y;
		w[11] = y;

		// Guardo w * w^T en la parte superior de A
		for( i=0; i<4; i++ ) A[i][i] = 0.0;
		for( i=8; i<12; i++ ) A[i][i] = 0.0;
		for( i=4; i<12; i++ ) {
			val = w[i];
			A[i][i] = val*val;
			for( j=i+1; j<12; j++ ) {
				A[i][j] = val*w[j];
			}
		}
		// Suma A a B
		for( i=0; i<12; i++ ) {
			tmp1 = &A[i][i];
			tmp2 = &B[i][i];
			for( j=i; j<12; j++ )
				*tmp2++ += *tmp1++;
		}
		//---------------------------
		w[0] =  datos[k][0];
		w[1] =  datos[k][1];
		w[2] =  datos[k][2];
		w[3] = 1.0;
		w[8] = -datos[k][0]*x;
		w[9] = -datos[k][1]*x;
		w[10] = -datos[k][2]*x;
		w[11] = -x;

		// Guardo w * w^T en la parte inferior de A
		for( i=0; i<4; i++ ) {
			val = w[i];
			A[i][i] = val*val;
			for( j=i+1; j<4; j++ ) {
				A[j][i] = val*w[j];
			}
			for( j=8; j<12; j++ ) {
				A[j][i] = val*w[j];
			}
		}
		for( i=4; i<8; i++ ) A[i][i] = 0.0;

		for( i=8; i<12; i++ ) {
			val = w[i];
			A[i][i] = val*val;
			for( j=i+1; j<12; j++ ) {
				A[j][i] = val*w[j];
			}
		}
		for( i=0; i<12; i++ ) {
			tmp2 = &B[i][i];
			for( j=i; j<12; j++ )
				*tmp2++ += A[j][i];
				// B[i][j] += A[j][i];
		}
	}
	// imprime( B, 12 );
	for( i=0; i<12; i++ ) {
		for( j=i+1; j<12; j++ )
			B[j][i] = B[i][j];
	}
	// Despus de aqu no se usar el puntero 'A'.
	// Aqui ya esta hecha la matrix B.
	// Se hace una copia de la matrix B
	for( i=0; i<12; i++ ) {
		tmp1 = Bcopia[i];
		tmp2 = B[i];
		for( j=0; j<12; j++ ) {
			*tmp1++ = *tmp2++;
		}
	}

	imprime( B, 12 );
	tridiagonal( B, 12, vec );
	imprime( B, 12 );

	//  Se calculan els eigenvalor ms pequeo
	biseccion( B, 12, eigen, 1, 1, vec );

	// printf ("Iteraciones: %d\n", (int)eigen[0] );

	// Se prueba que el eigenvalor sea mayor que cero (mayor a 1e-8)
	val = eigen[1];
	if( fabs( val ) < 1e-8 ){
		fprintf( stderr, "ERROR: Los eigenvalores seleccionados son todos iguales a cero\n" );
		matrix_free( M2 );
		matrix_free( M3 );
		matrix_free( M4 );
		free(vec);
		exit(2);
	}

	/** Se calcula el eigenvector asociado al eigenvalor encontrado
	    Se usa la matriz original y la descomposicin QR  **/

	// printf( "e: %lf\n", eigen[cual] );

	for( i=0; i<12; i++ ) {
		Bcopia[i][12] = 0.288675134595;   //  #  1/sqrt(12)
		Bcopia[i][i] -= val;
	}
	// printf( "Hice copia\n" );
	descomposicionQR( Bcopia, 12, 13, R, vec );

	// printf( "R:\n" );
	// for( i=0; i<n; i++ ) {
	// 	for( j=0; j<=n; j++ ) {
	// 		printf ("%d %d %lf\n", i, j, R[i][j] );
	// 	}
	// 	printf ("\n");
	// }
	// printf ("\n");

	resuelve( R, 12, vec );

	// printf ("r: ");
	// for( i=0; i<n; i++ ) {
	// 	printf( "%lf ", vec[i] );
	// }
	// printf("\n");
	
	suma=0.0;
	tmp1 = vec;
	for( i=0; i<12; i++ ) {
		val = *tmp1++;
		suma += val*val;
	}
	suma = sqrt( suma );
	tmp1 = vec;
	for( i=0; i<12; i++ ) {
		*tmp1++ /= suma;
	}
	

  }

	gettimeofday(&tv2, NULL);
	printf ("Total time = %f seconds\n",
	     (double) (tv2.tv_usec - tv1.tv_usec) / 1000000 +
	     (double) (tv2.tv_sec - tv1.tv_sec));

	printf( "e: %g\n", eigen[1] );
	// for( i=1; i<=3; i++ ) {
	// 	printf ("e(%d) = %g\n", i, eigen[i] );
	// }
	printf("rnorm: ");
	for( i=0; i<12; i++ ) {
		printf ("%g ", vec[i] );
	}
	printf("\n");

	matrix_free( M2 );
	matrix_free( M3 );
	matrix_free( M4 );
	free( vec );

	return 0;
}

